@@ -10,24 +10,25 @@ from qtpy.QtCore import Signal
 from .. import actions
 from .. import cmds
 from .. import core
+from .. import display
 from .. import gitcmds
 from .. import hotkeys
 from .. import icons
 from .. import textwrap
 from .. import qtutils
 from .. import spellcheck
+from .. import utils
 from ..interaction import Interaction
 from ..gitcmds import commit_message_path
 from ..i18n import N_
 from ..models import dag
 from ..models import prefs
 from ..qtutils import get
-from ..utils import Group
 from . import defs
 from . import standard
 from .selectcommits import select_commits
 from .spellcheck import SpellCheckLineEdit, SpellCheckTextEdit
-from .text import anchor_mode
+from .text import event_anchor_mode, is_shift_pressed
 
 
 class CommitMessageEditor(QtWidgets.QFrame):
@@ -43,7 +44,9 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.model = model = context.model
         self.spellcheck_initialized = False
         self.spellcheck = spellcheck.NorvigSpellCheck()
-        self.spellcheck.set_dictionary(cfg.get('cola.dictionary', None))
+        self.spellcheck.add_dictionaries(cfg.get_all('cola.dictionary'))
+        self.spellcheck.set_aspell_enabled(prefs.aspell_enabled(context))
+        self.spellcheck.set_aspell_langs(cfg.get_all('cola.aspell.lang'))
 
         self._linebreak = None
         self._textwidth = None
@@ -51,6 +54,11 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self._last_commit_datetime = None  # The most recently selected commit date.
         self._last_commit_datetime_backup = None  # Used when amending.
         self._git_commit_date = None  # Overrides the commit date when committing.
+        self._commit_authors = []  # Recently used author override values.
+        self._last_git_commit_author = None  # The most recently selected author.
+        self._git_commit_author = None  # Overrides the commit author when comitting.
+        self._git_commit_author_backup = None  # Used when amending.
+
         self._widgets_initialized = False  # Defer setting the cursor position height.
 
         # Actions
@@ -103,7 +111,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.commit_button = qtutils.create_button(
             text=N_('Commit@@verb'), tooltip=commit_button_tooltip, icon=icons.commit()
         )
-        self.commit_group = Group(self.commit_action, self.commit_button)
+        self.commit_group = utils.Group(self.commit_action, self.commit_button)
         self.commit_progress_bar = standard.progress_bar(
             self,
             disable=(self.commit_button, self.summary, self.description),
@@ -129,11 +137,17 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.actions_menu.addSeparator()
 
         # Amend checkbox
-        self.amend_action = self.actions_menu.addAction(N_('Amend Last Commit'))
+        self.amend_action = qtutils.add_action_bool(
+            self,
+            N_('Amend Last Commit'),
+            partial(cmds.run(cmds.AmendMode), context),
+            False,
+            *hotkeys.AMEND,
+        )
         self.amend_action.setIcon(icons.edit())
         self.amend_action.setCheckable(True)
-        self.amend_action.setShortcuts(hotkeys.AMEND)
         self.amend_action.setShortcutContext(Qt.ApplicationShortcut)
+        self.actions_menu.addAction(self.amend_action)
 
         # Commit Date
         self.commit_date_action = self.actions_menu.addAction(N_('Set Commit Date'))
@@ -141,6 +155,12 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.commit_date_action.setChecked(False)
         qtutils.connect_action_bool(self.commit_date_action, self.set_commit_date)
 
+        # Commit Author
+        self.commit_author_action = self.actions_menu.addAction(N_('Set Commit Author'))
+        self.commit_author_action.setCheckable(True)
+        self.commit_author_action.setChecked(False)
+        qtutils.connect_action_bool(self.commit_author_action, self.set_commit_author)
+
         # Bypass hooks
         self.bypass_commit_hooks_action = self.actions_menu.addAction(
             N_('Bypass Commit Hooks')
@@ -176,6 +196,39 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.fixup_commit_menu = self.actions_menu.addMenu(N_('Fixup Previous Commit'))
         self.fixup_commit_menu.aboutToShow.connect(self.build_fixup_menu)
 
+        # Display commit author overrides.
+        self._author_close = qtutils.create_action_button(
+            icon=icons.close(),
+            tooltip=N_('Cancel the commit author override.'),
+        )
+        tooltip = N_('A commit will be recorded with this author')
+        self._author_image = qtutils.pixmap_label(
+            icons.person(), defs.default_icon, tooltip=tooltip, parent=self
+        )
+        self._author_label = qtutils.plain_text_label(tooltip=tooltip, parent=self)
+        tooltip = N_('A commit will be recorded with this date and time')
+
+        # Display commit date overrides.
+        self._date_close = qtutils.create_action_button(
+            icon=icons.close(),
+            tooltip=N_('Cancel the commit date override.'),
+        )
+        self._date_image = qtutils.pixmap_label(
+            icons.clock(), defs.default_icon, tooltip=tooltip, parent=self
+        )
+        self._date_label = qtutils.plain_text_label(tooltip=tooltip, parent=self)
+
+        self.bottomlayout = qtutils.hbox(
+            defs.no_margin,
+            defs.spacing,
+            self._author_close,
+            self._author_image,
+            self._author_label,
+            qtutils.STRETCH,
+            self._date_close,
+            self._date_image,
+            self._date_label,
+        )
         self.toplayout = qtutils.hbox(
             defs.no_margin,
             defs.spacing,
@@ -188,19 +241,19 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.topwidget = QtWidgets.QWidget()
         self.topwidget.setLayout(self.toplayout)
 
-        self.mainlayout = qtutils.vbox(defs.no_margin, defs.spacing, self.description)
+        self.mainlayout = qtutils.vbox(
+            defs.no_margin, defs.spacing, self.description, self.bottomlayout
+        )
         self.setLayout(self.mainlayout)
 
         qtutils.connect_button(self.commit_button, self.commit)
-
-        # Broadcast the amend mode
-        qtutils.connect_action_bool(
-            self.amend_action, partial(cmds.run(cmds.AmendMode), context)
+        qtutils.connect_button(
+            self._author_close, lambda: self.set_commit_author(False)
         )
+        qtutils.connect_button(self._date_close, lambda: self.set_commit_date(False))
         qtutils.connect_action_bool(
             self.check_spelling_action, self.toggle_check_spelling
         )
-
         # Handle the one-off auto-wrapping
         qtutils.connect_action_bool(self.autowrap_action, self.set_linebreak)
 
@@ -256,6 +309,18 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.setMaximumHeight(133)
         QtCore.QTimer.singleShot(1, self.restore_size)
 
+    def export_state(self, state):
+        """Save persistent UI state on shutdown"""
+        # Set a limit on the number of recent author values.
+        max_recent = prefs.maxrecent(self.context)
+        state['authors'] = self._commit_authors[:max_recent]
+        return state
+
+    def apply_state(self, state):
+        """Apply persistent UI state on startup"""
+        self._commit_authors = state.get('authors', [])
+        return True
+
     def restore_size(self):
         self.setMaximumHeight(2**13)
 
@@ -271,7 +336,6 @@ class CommitMessageEditor(QtWidgets.QFrame):
         If the cursor is at the end of the line then focus the description.
         Otherwise, move the cursor to the end of the line so that a
         subsequence "down" press moves to the end of the line.
-
         """
         self.focus_description()
 
@@ -411,13 +475,59 @@ class CommitMessageEditor(QtWidgets.QFrame):
         with qtutils.BlockSignals(self.amend_action):
             self.amend_action.setEnabled(can_amend)
             self.amend_action.setChecked(checked)
-        # Store/restore the last commit date when amending.
+        # Store/restore the commit date and author when amending.
         if checked:
             self._last_commit_datetime_backup = self._last_commit_datetime
             self._last_commit_datetime = _get_latest_commit_datetime(self.context)
+            self._git_commit_author_backup = self._git_commit_author
+            self._git_commit_author = None
         else:
             self._last_commit_datetime = self._last_commit_datetime_backup
             self._last_commit_datetime_backup = None
+            if self._git_commit_author_backup:
+                self._git_commit_author = self._git_commit_author_backup
+                self._git_commit_author_backup = None
+                with qtutils.BlockSignals(self.commit_author_action):
+                    self.commit_author_action.setChecked(True)
+
+        self.update_author_and_date()
+
+    def update_author_and_date(self):
+        """Hide or display the author field"""
+        amending = self.model.mode == self.model.mode_amend
+        # Display the author when amending or when overridden.
+        if self._git_commit_author:
+            author = self._git_commit_author
+        elif amending:
+            author = self.model.commit_author
+        else:
+            author = None
+        author_visible = bool(author)
+        if author:
+            self._author_label.setText(author)
+        author_checked = self.commit_author_action.isChecked()
+        self._author_close.setVisible(author_visible and author_checked)
+        self._author_image.setVisible(author_visible)
+        self._author_label.setVisible(author_visible)
+
+        # Display the commit date when amending or when overridden.
+        if self._git_commit_date:
+            date = self._git_commit_date
+        elif amending and self._last_commit_datetime:
+            date = display.git_commit_date(self._last_commit_datetime)
+        else:
+            date = ''
+        date_visible = bool(date)
+        if date:
+            self._date_label.setText(date)
+        self._date_close.setVisible(bool(self._git_commit_date))
+        self._date_image.setVisible(date_visible)
+        self._date_label.setVisible(date_visible)
+
+        if author_visible or date_visible:
+            self.actions_button.setIcon(icons.gear_solid())
+        else:
+            self.actions_button.setIcon(icons.configure())
 
     def commit(self):
         """Attempt to create a commit from the index and commit message."""
@@ -487,13 +597,16 @@ class CommitMessageEditor(QtWidgets.QFrame):
 
         sign = get(self.sign_action)
         no_verify = get(self.bypass_commit_hooks_action)
-        self.bypass_commit_hooks_action.setChecked(False)
         if self.commit_date_action.isChecked():
-            self.commit_date_action.setChecked(False)
             date = self._git_commit_date
         else:
             date = None
 
+        if self.commit_author_action.isChecked():
+            author = self._git_commit_author
+        else:
+            author = None
+
         task = qtutils.SimpleTask(
             cmds.run(
                 cmds.Commit,
@@ -502,6 +615,7 @@ class CommitMessageEditor(QtWidgets.QFrame):
                 msg,
                 sign,
                 no_verify=no_verify,
+                author=author,
                 date=date,
             )
         )
@@ -516,6 +630,12 @@ class CommitMessageEditor(QtWidgets.QFrame):
         title = N_('Commit failed')
         status, out, err = task.result
         Interaction.command(title, 'git commit', status, out, err)
+        self.bypass_commit_hooks_action.setChecked(False)
+        # Author and date settings are not cleared unless the commit operation succeeds.
+        # These settings are consumed when a commit is produced with their values.
+        if status == 0:
+            self.set_commit_author(False, update=False)
+            self.set_commit_date(False, update=True)
         self.setFocus()
 
     def build_fixup_menu(self):
@@ -628,10 +748,15 @@ class CommitMessageEditor(QtWidgets.QFrame):
         self.cursor_position_label.setStyleSheet(stylesheet)
         self.cursor_position_label.setText(display_content)
 
-    def set_commit_date(self, enabled):
+    def set_commit_date(self, enabled, update=True):
         """Choose the date and time that is used when authoring commits"""
         if not enabled:
             self._git_commit_date = None
+            if self.commit_date_action.isChecked():
+                with qtutils.BlockSignals(self.commit_date_action):
+                    self.commit_date_action.setChecked(False)
+            if update:
+                self.update_author_and_date()
             return
         widget = CommitDateDialog(
             self, self.context, commit_datetime=self._last_commit_datetime
@@ -643,6 +768,43 @@ class CommitMessageEditor(QtWidgets.QFrame):
             self._last_commit_datetime = CommitDateDialog.tick_time(widget.datetime())
         else:
             self.commit_date_action.setChecked(False)
+        if update:
+            self.update_author_and_date()
+
+    def set_commit_author(self, enabled, update=True):
+        """Choose a commit author to override the author value when authoring commits"""
+        if not enabled:
+            if self._git_commit_author:
+                self._last_git_commit_author = self._git_commit_author
+            self._git_commit_author = None
+            if self.commit_author_action.isChecked():
+                with qtutils.BlockSignals(self.commit_author_action):
+                    self.commit_author_action.setChecked(False)
+            if update:
+                self.update_author_and_date()
+            return
+        widget = CommitAuthorDialog(
+            self,
+            self.context,
+            commit_author=self._git_commit_author or self._last_git_commit_author,
+            commit_authors=self._commit_authors,
+        )
+        if widget.exec_() == QtWidgets.QDialog.Accepted:
+            commit_author = widget.commit_author()
+            default_author = _get_default_author(self.context)
+            Interaction.log(N_('Setting commit author to %s') % commit_author)
+            self._git_commit_author = commit_author
+            if (
+                commit_author
+                and commit_author != default_author
+                and commit_author not in self._commit_authors
+            ):
+                self._commit_authors.insert(0, commit_author)
+        else:
+            self._git_commit_author = None
+            self.commit_author_action.setChecked(False)
+        if update:
+            self.update_author_and_date()
 
     # Qt overrides
     def showEvent(self, event):
@@ -679,64 +841,69 @@ class CommitDateDialog(QtWidgets.QDialog):
         QtWidgets.QDialog.__init__(self, parent)
         slider_range = self.slider_range
         self.context = context
-        self._calendar_widget = calendar_widget = QtWidgets.QCalendarWidget()
-        self._time_widget = time_widget = QtWidgets.QTimeEdit()
-        time_widget.setDisplayFormat('hh:mm:ss AP')
+        self._calendar_widget = QtWidgets.QCalendarWidget()
+        self._time_widget = QtWidgets.QTimeEdit()
+        self._time_widget.setDisplayFormat('hh:mm:ss AP')
 
         # Horizontal slider moves the date and time backwards and forwards.
-        self._slider = slider = QtWidgets.QSlider(Qt.Horizontal)
-        slider.setRange(0, slider_range)  # Mapped from 00:00:00 to 23:59:59
+        self._slider = QtWidgets.QSlider(Qt.Horizontal)
+        self._slider.setRange(0, slider_range)  # Mapped from 00:00:00 to 23:59:59
 
-        self._tick_backward = tick_backward = qtutils.create_toolbutton_with_callback(
+        self._tick_backward = qtutils.create_toolbutton_with_callback(
             partial(self._adjust_slider, -1),
             '-',
             None,
             N_('Decrement'),
             repeat=True,
         )
-        self._tick_forward = tick_forward = qtutils.create_toolbutton_with_callback(
+        self._tick_forward = qtutils.create_toolbutton_with_callback(
             partial(self._adjust_slider, 1),
             '+',
             None,
             N_('Increment'),
             repeat=True,
         )
-        self._reset_to_commit = (
-            reset_to_commit
-        ) = qtutils.create_toolbutton_with_callback(
-            self._reset_commit_time,
+        self._reset_to_commit_time = qtutils.create_toolbutton_with_callback(
+            self._reset_time_to_latest_commit,
             None,
             icons.sync(),
             N_('Reset time to latest commit'),
         )
+        self._reset_to_current_time = qtutils.create_toolbutton_with_callback(
+            lambda: self._reset_time_to_datetime(datetime.datetime.now()),
+            None,
+            icons.style_dialog_reset(),
+            N_('Reset time to current time'),
+        )
 
-        cancel_button = QtWidgets.QPushButton(N_('Cancel'))
-        cancel_button.setIcon(icons.close())
+        self._cancel_button = QtWidgets.QPushButton(N_('Cancel'))
+        self._cancel_button.setIcon(icons.close())
 
-        set_commit_time_button = QtWidgets.QPushButton(N_('Set Date and Time'))
-        set_commit_time_button.setDefault(True)
-        set_commit_time_button.setIcon(icons.ok())
+        self._set_commit_time_button = QtWidgets.QPushButton(N_('Set Date and Time'))
+        self._set_commit_time_button.setDefault(True)
+        self._set_commit_time_button.setIcon(icons.ok())
 
         button_layout = qtutils.hbox(
             defs.no_margin,
             defs.button_spacing,
-            cancel_button,
+            self._cancel_button,
             qtutils.STRETCH,
-            set_commit_time_button,
+            self._set_commit_time_button,
         )
         slider_layout = qtutils.hbox(
             defs.no_margin,
             defs.spacing,
-            tick_backward,
-            slider,
-            tick_forward,
-            reset_to_commit,
-            time_widget,
+            self._tick_backward,
+            self._slider,
+            self._tick_forward,
+            self._reset_to_commit_time,
+            self._reset_to_current_time,
+            self._time_widget,
         )
         layout = qtutils.vbox(
             defs.small_margin,
             defs.spacing,
-            calendar_widget,
+            self._calendar_widget,
             slider_layout,
             defs.button_spacing,
             button_layout,
@@ -747,8 +914,8 @@ class CommitDateDialog(QtWidgets.QDialog):
 
         if commit_datetime is None:
             commit_datetime = self.tick_time(_get_latest_commit_datetime(context))
-        time_widget.setTime(commit_datetime.time())
-        calendar_widget.setSelectedDate(commit_datetime.date())
+        self._time_widget.setTime(commit_datetime.time())
+        self._calendar_widget.setSelectedDate(commit_datetime.date())
         self._update_slider_from_datetime(commit_datetime)
 
         self._right_action = qtutils.add_action(
@@ -758,12 +925,12 @@ class CommitDateDialog(QtWidgets.QDialog):
             self, N_('Decrement'), partial(self._adjust_slider, -1), hotkeys.CTRL_LEFT
         )
 
-        time_widget.timeChanged.connect(self._update_slider_from_time_signal)
-        slider.valueChanged.connect(self._update_time_from_slider)
-        calendar_widget.activated.connect(lambda _: self.accept())
+        self._time_widget.timeChanged.connect(self._update_slider_from_time_signal)
+        self._slider.valueChanged.connect(self._update_time_from_slider)
+        self._calendar_widget.activated.connect(lambda _: self.accept())
 
-        cancel_button.clicked.connect(self.reject)
-        set_commit_time_button.clicked.connect(self.accept)
+        self._cancel_button.clicked.connect(self.reject)
+        self._set_commit_time_button.clicked.connect(self.accept)
 
     @classmethod
     def tick_time(cls, commit_datetime):
@@ -790,7 +957,7 @@ class CommitDateDialog(QtWidgets.QDialog):
 
     def commit_date(self):
         """Return the selected datetime as a string for use by Git"""
-        return self.datetime().strftime('%a %b %d %H:%M:%S %Y %z')
+        return display.git_commit_date(self.datetime())
 
     def _update_time_from_slider(self, value):
         """Map the slider value to an offset corresponding to the current time.
@@ -803,9 +970,8 @@ class CommitDateDialog(QtWidgets.QDialog):
         delta = datetime.timedelta(seconds=int(ratio * seconds_range))
         midnight = datetime.datetime(1999, 12, 31)
         new_time = (midnight + delta).time()
-        time_widget = self._time_widget
-        with qtutils.BlockSignals(time_widget):
-            time_widget.setTime(new_time)
+        with qtutils.BlockSignals(self._time_widget):
+            self._time_widget.setTime(new_time)
 
     def _adjust_slider(self, amount):
         """Adjust the slider forward or backwards"""
@@ -833,9 +999,13 @@ class CommitDateDialog(QtWidgets.QDialog):
         with qtutils.BlockSignals(self._slider):
             self._slider.setValue(value)
 
-    def _reset_commit_time(self):
+    def _reset_time_to_latest_commit(self):
         """Reset the commit time to match the most recent commit"""
         commit_datetime = _get_latest_commit_datetime(self.context)
+        self._reset_time_to_datetime(commit_datetime)
+
+    def _reset_time_to_datetime(self, commit_datetime):
+        """Reset the commit time to match the specified datetime"""
         with qtutils.BlockSignals(self._time_widget):
             self._time_widget.setTime(commit_datetime.time())
         with qtutils.BlockSignals(self._calendar_widget):
@@ -843,6 +1013,143 @@ class CommitDateDialog(QtWidgets.QDialog):
         self._update_slider_from_datetime(commit_datetime)
 
 
+def _get_default_author(context):
+    """Get the default author value"""
+    name, email = context.cfg.get_author()
+    return f'{name} <{email}>'
+
+
+def _get_latest_commit_author(context):
+    """Query the commit author from Git"""
+    status, out, _ = context.git.log('-1', '--format=%aN <%aE>', 'HEAD')
+    if status != 0 or not out:
+        return None
+    return out
+
+
+class CommitAuthorDialog(QtWidgets.QDialog):
+    """Override the commit author when authoring commits"""
+
+    def __init__(self, parent, context, commit_author=None, commit_authors=None):
+        QtWidgets.QDialog.__init__(self, parent)
+        main_tooltip = """Override the commit author.
+
+Specify an explicit author using the standard "A U Thor <author@example.com>" format.
+
+Otherwise <author> is assumed to be a pattern and is used to search for an
+existing commit by that author (i.e. rev-list --all -i --author=<author>);"""
+        self.context = context
+        self._current_author = commit_author or _get_default_author(context)
+
+        authors = self._get_authors(commit_author, commit_authors)
+        tooltip = N_('Override the commit author when authoring commits')
+        self._author_combobox = qtutils.combo(authors, editable=True, tooltip=tooltip)
+        self._author_combobox.setToolTip(main_tooltip)
+        if authors:
+            self._author_combobox.set_value(authors[0])
+
+        self._reset_to_commit_author_button = qtutils.create_toolbutton_with_callback(
+            self._reset_to_commit_author,
+            None,
+            icons.sync(),
+            N_('Set author to match the latest commit'),
+        )
+        self._reset_to_current_author_button = qtutils.create_toolbutton_with_callback(
+            self._reset_to_current_author,
+            None,
+            icons.undo(),
+            N_('Reset author to the current author'),
+        )
+        self._reset_to_default_author_button = qtutils.create_toolbutton_with_callback(
+            self._reset_to_default_author,
+            None,
+            icons.style_dialog_reset(),
+            N_('Reset to default author'),
+        )
+
+        self._cancel_button = QtWidgets.QPushButton(N_('Cancel'))
+        self._cancel_button.setIcon(icons.close())
+
+        self._apply_button = QtWidgets.QPushButton(N_('Set Author'))
+        self._apply_button.setToolTip(main_tooltip)
+        self._apply_button.setDefault(True)
+        self._apply_button.setIcon(icons.ok())
+
+        button_layout = qtutils.hbox(
+            defs.no_margin,
+            defs.button_spacing,
+            self._cancel_button,
+            qtutils.STRETCH,
+            self._apply_button,
+        )
+        input_layout = qtutils.hbox(
+            defs.no_margin,
+            defs.spacing,
+            self._author_combobox,
+            self._reset_to_commit_author_button,
+            self._reset_to_default_author_button,
+            self._reset_to_current_author_button,
+        )
+        layout = qtutils.vbox(
+            defs.small_margin,
+            defs.spacing,
+            input_layout,
+            qtutils.STRETCH,
+            defs.button_spacing,
+            button_layout,
+        )
+        self.setLayout(layout)
+        self.setWindowTitle(N_('Set Commit Author'))
+        self.setWindowModality(Qt.ApplicationModal)
+
+        self._apply_button.clicked.connect(self.accept)
+        self._cancel_button.clicked.connect(self.reject)
+        self._author_combobox.currentTextChanged.connect(lambda _: self._validate())
+        self._validate()
+
+    def commit_author(self):
+        """Return the selected author value"""
+        return self._author_combobox.current_value().strip()
+
+    def _validate(self):
+        """Validate the author value and disable the apply button when invalid"""
+        author = self.commit_author()
+        self._apply_button.setEnabled(bool(author))
+
+    def _get_authors(self, commit_author, commit_authors):
+        """Build a list of authors for the combo box"""
+        seen = set()
+        all_authors = [commit_author]
+        if commit_authors:
+            all_authors.extend(commit_authors)
+        all_authors.append(_get_default_author(self.context))
+        # Create a final unique list of authors.
+        authors = []
+        for author in all_authors:
+            if author and author not in seen:
+                seen.add(author)
+                authors.append(author)
+        return authors
+
+    def _reset_to_commit_author(self):
+        """Reset the author value to the author of the most recent commit"""
+        commit_author = _get_latest_commit_author(self.context)
+        if commit_author:
+            self._author_combobox.set_current_value(commit_author)
+        else:
+            self._reset_to_default_author()  # Fallback to the current author.
+
+    def _reset_to_current_author(self):
+        """Reset the author value to the current author"""
+        current_author = self._current_author
+        if current_author:
+            self._author_combobox.set_current_value(current_author)
+
+    def _reset_to_default_author(self):
+        """Reset the author value to the default author"""
+        self._author_combobox.set_current_value(_get_default_author(self.context))
+
+
 class CommitSummaryLineEdit(SpellCheckLineEdit):
     """Text input field for the commit summary"""
 
@@ -864,8 +1171,8 @@ class CommitSummaryLineEdit(SpellCheckLineEdit):
 
     def _update_summary_text(self):
         """Prevent commit messages from starting with comment characters"""
-        value = self.value()
-        if self._comment_char and value.startswith(self._comment_char):
+        value = self.get()
+        if self._comment_char and value.lstrip().startswith(self._comment_char):
             cursor = self.textCursor()
             position = cursor.position()
 
@@ -874,10 +1181,10 @@ class CommitSummaryLineEdit(SpellCheckLineEdit):
                 value = value.lstrip(self._comment_char).lstrip()
 
             self.set_value(value, block=True)
+            value = self.get()
 
-            value = self.value()
             if position > 1:
-                position = max(0, min(position - 1, len(value) - 1))
+                position = utils.clamp_zero(position - 1, len(value))
                 cursor.setPosition(position)
                 self.setTextCursor(cursor)
 
@@ -905,15 +1212,17 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
         )
 
     def keyPressEvent(self, event):
+        """Update the cursor display and move the cursor to the end and beginning"""
         if event.key() == Qt.Key_Up:
             cursor = self.textCursor()
             position = cursor.position()
             if position == 0:
                 # The cursor is at the beginning of the line.
-                # If we have selection then simply reset the cursor.
-                # Otherwise, emit a signal so that the parent can
-                # change focus.
-                if cursor.hasSelection():
+                # If we have a selection and shift is not held then reset the
+                # cursor to clear the selection. Otherwise, emit a signal so
+                # that the parent can change focus.
+                shifted = is_shift_pressed(event)
+                if cursor.hasSelection() and not shifted:
                     self.set_cursor_position(0)
                 else:
                     self.leave.emit()
@@ -925,11 +1234,11 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
                 # If we're on the first line, but not at the
                 # beginning, then move the cursor to the beginning
                 # of the line.
-                if event.modifiers() & Qt.ShiftModifier:
-                    mode = QtGui.QTextCursor.KeepAnchor
-                else:
-                    mode = QtGui.QTextCursor.MoveAnchor
-                cursor.setPosition(0, mode)
+                mode = event_anchor_mode(event)
+                cursor.movePosition(QtGui.QTextCursor.Up, mode)
+                new_position = cursor.position()
+                if position == new_position:
+                    cursor.setPosition(0, mode)
                 self.setTextCursor(cursor)
                 event.accept()
                 return
@@ -940,9 +1249,11 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
             text_after = all_text[position:]
             lines_after = text_after.count('\n')
             if lines_after == 0:
-                select = event.modifiers() & Qt.ShiftModifier
-                mode = anchor_mode(select)
-                cursor.setPosition(len(all_text), mode)
+                mode = event_anchor_mode(event)
+                cursor.movePosition(QtGui.QTextCursor.Down, mode)
+                new_position = cursor.position()
+                if position == new_position:
+                    cursor.setPosition(len(all_text), mode)
                 self.setTextCursor(cursor)
                 event.accept()
                 return
